DROP SCHEMA public;
CREATE SCHEMA public;
ALTER SCHEMA public OWNER TO postgres;

-- -----------------------------------------------------
-- Name: order; Type: TABLE; Schema: public;
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS "order" (
   o_id           integer                  GENERATED BY DEFAULT AS IDENTITY,
   total          NUMERIC(12,2)         default 0,
   CONSTRAINT PK_ORDER primary key (o_id)
);

-- -----------------------------------------------------
-- Name: order_item; Type: TABLE; Schema: public;
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS order_item (  
   oi_id                integer                  GENERATED BY DEFAULT AS IDENTITY,
   order_id             integer                  ,
   product_id           integer                  NOT NULL,
   price            NUMERIC(12,2)        NOT NULL default 0,
   CONSTRAINT PK_ORDERITEM primary key (oi_id)
);

-- -----------------------------------------------------
-- Name: product; Type: TABLE; Schema: public;
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS product (
    p_id       integer                     GENERATED BY DEFAULT AS IDENTITY,
    name       CHARACTER VARYING(200)           NOT NULL,
    CONSTRAINT PK_PRODUCT PRIMARY KEY (p_id)
);

alter table order_item
   add CONSTRAINT FK_ORDERITE_REFERENCE_ORDER foreign key (order_id)
      references "order" (o_id);

ALTER TABLE order_item 
    ADD CONSTRAINT FK_PRODUCT_ITEM FOREIGN KEY (product_id)
        REFERENCES "product" (p_id);

-- -----------------------------------------------------
-- Name: actor; Type: TABLE; Schema: public;
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS actor (
    a_id    integer GENERATED BY DEFAULT AS IDENTITY,
    name    CHARACTER VARYING(200)   NOT NULL,
    photo   bytea,
    CONSTRAINT PK_ACTOR PRIMARY KEY (a_id)
);

-- -----------------------------------------------------
-- Name: movie; Type: TABLE; Schema: public;
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS movie (
    m_id    integer GENERATED BY DEFAULT AS IDENTITY,
    name CHARACTER VARYING(200) NOT NULL,
    file   bytea,
    CONSTRAINT PK_MOVIE PRIMARY  KEY (m_id)
);

-- -----------------------------------------------------
-- Name: actor_movie; Type: TABLE; Schema: public;
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS actor_movie (
    a_id integer NOT NULL,
    m_id integer NOT NULL,
    
    CONSTRAINT PK_ACTOR_MOVIE PRIMARY KEY (a_id, m_id)
);

ALTER TABLE actor_movie 
    ADD CONSTRAINT FK_ACTOR FOREIGN KEY (a_id)
        REFERENCES "actor" (a_id);
		
ALTER TABLE actor_movie 
    ADD CONSTRAINT FK_MOVIE FOREIGN KEY (m_id)
        REFERENCES "movie" (m_id);      

-- -----------------------------------------------------
-- Name: groups1; Type: TABLE; Schema: public;
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS groups1 (
   g_id   integer GENERATED BY DEFAULT AS IDENTITY,
   name CHARACTER VARYING(20) NOT NULL,
   parent_group_id integer ,
   CONSTRAINT PK_GROUPS1 PRIMARY KEY (g_id)
);

ALTER TABLE groups1 
    ADD CONSTRAINT FK_PARENT_GROUP FOREIGN KEY (parent_group_id)
        REFERENCES groups1 (g_id);

-- -----------------------------------------------------
-- Name: orders2; Type: TABLE; Schema: public;
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS orders2 (
   o_id           integer                  GENERATED BY DEFAULT AS IDENTITY,
   customer_vatid         integer,
   CONSTRAINT PK_ORDER2 primary key (o_id)
);

-- -----------------------------------------------------
-- Name: customers2; Type: TABLE; Schema: public;
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS customers2 (
   c_id       integer GENERATED BY DEFAULT AS IDENTITY,
   vatid      integer NOT NULL,
   CONSTRAINT PK_CUSTOMERS2 primary key (c_id)
);

-- -----------------------------------------------------
-- Name: C2_VAT_ID; Type: INDEX; Schema: public;
-- -----------------------------------------------------
CREATE UNIQUE INDEX C2_VAT_ID ON customers2 (vatid);

ALTER TABLE orders2 
    ADD CONSTRAINT FK_ORDERS_VAT_ID FOREIGN KEY (customer_vatid)
        REFERENCES customers2 (vatid);
