﻿using System;
using System.IO;
using FastTests.Voron;
using Voron.Impl.Journal;
using Xunit;
using Xunit.Abstractions;

namespace SlowTests.Voron
{
    public class RavenDB_11871 : StorageTest
    {
        public RavenDB_11871(ITestOutputHelper output) : base(output)
        {
        }

        [Fact]
        public void WillNotRetainJournalsAfterSync()
        {
            RequireFileBasedPager();

            Options.ManualFlushing = true;
            Options.MaxLogFileSize = 1024 * 1024;
            Options.ManualSyncing = true;

            for (int X = 0; X < 3; X++)
            {
                using (var tx = Env.WriteTransaction())
                {
                    var tree = tx.CreateTree("t");

                    for (int i = 0; i < 50_000; i++)
                    {
                        tree.Add(i.ToString() + "-" + X, Guid.NewGuid().ToByteArray());
                    }

                    tx.Commit();
                }
            }

            Env.FlushLogToDataFile();

            var journalsDir = Path.Combine(DataDir, "Journals");

            Assert.NotEmpty(Directory.GetFiles(journalsDir, "*.journal"));

            using (var tx = Env.WriteTransaction())
            {
                var tree = tx.CreateTree("t");

                for (int i = 0; i < 50_000; i++)
                {
                    tree.Add(i.ToString() + "-5", Guid.NewGuid().ToByteArray());
                }

                tx.Commit();
            }

            using (var op = new WriteAheadJournal.JournalApplicator.SyncOperation(Env.Journal.Applicator))
            {
                op.SyncDataFile();
            }
            Env.FlushLogToDataFile();

            using (var op = new WriteAheadJournal.JournalApplicator.SyncOperation(Env.Journal.Applicator))
            {
                op.SyncDataFile();
            }

            Assert.Empty(Directory.GetFiles(journalsDir, "*.journal"));
        }
    }
}
